#include <WiFi.h>
#include <WebServer.h>
#include <ESPmDNS.h>
#include <ArduinoOTA.h>
#include <FastLED.h>
#include <FS.h>
#include <SPIFFS.h>

// WLAN-Zugangsdaten
const char* ssid = "ESP32";
const char* password = "ESP32felix";

// Softwareversion
const char* SOFTWARE_VERSION = "V0.11";

// Webserver auf Port 80
WebServer server(80);

// Pin-Definitionen für DRV8833 1 (linke Motoren)
#define IN1_LEFT 12
#define IN2_LEFT 13
#define IN3_LEFT 14
#define IN4_LEFT 15

// Pin-Definitionen für DRV8833 2 (rechte Motoren)
#define IN1_RIGHT 18
#define IN2_RIGHT 19
#define IN3_RIGHT 21
#define IN4_RIGHT 22

// Pin-Definitionen für WS2812-LEDs
#define LED_PIN_FRONT 26
#define LED_PIN_REAR 27

// Pin-Definitionen für HC-SR04
#define TRIG_PIN 33
#define ECHO_PIN 32

// Globale Variable für Geschwindigkeit
int motorSpeed = 255; // PWM-Wert für Geschwindigkeit (0-255)

// Globale Variablen für Hinderniserkennung
bool obstacleDetected = false;
const float OBSTACLE_DISTANCE = 25.0; // Schwellenwert in cm
const int OBSTACLE_CONFIRM_COUNT = 15; // Anzahl Messungen für Bestätigung
int obstacleCount = 0; // Zähler für aufeinanderfolgende Hindernismessungen

// Funktionsprototypen
void forward();
void backward();
void turnLeft();
void turnRight();
void stop();
void startBlinking(String direction);
void stopBlinking();
void setEmergencyBlink(bool state);
void initLeds();
void setLedColors();
void setLedBrightness(int brightness);
void setLightEffect(String effect);
void checkObstacle();
void handleRecorderRoutes();
void logRecorderCommand(String command);
void updateRecorder();

// HTML für das Webinterface mit Platzhalter
const char* html = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
  <title>Modellauto-Steuerung</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <style>
    body { font-family: Arial, sans-serif; text-align: center; margin: 20px; }
    button { padding: 15px 30px; font-size: 18px; }
    .button-container { margin: 10px 0; }
    .flex-container { display: flex; justify-content: center; gap: 20px; margin: 10px 0; }
    .slider { width: 50%; margin: 20px auto; }
    .version { font-size: 14px; color: #666; margin-top: 10px; }
    .dropdown-container { margin: 15px 0; }
    select { padding: 10px; font-size: 16px; }
    .warning { color: red; font-weight: bold; display: none; }
    .recorder-container { margin: 20px 0; }
    .recorder-container button { margin: 5px; }
  </style>
</head>
<body>
  <h1>Modellauto-Steuerung</h1>
  <div class="button-container">
    <button onmousedown="sendCommand('forward')" onmouseup="sendCommand('stop')" ontouchstart="sendCommand('forward')" ontouchend="sendCommand('stop')">Vor</button>
  </div>
  <div class="flex-container">
    <button onmousedown="sendCommand('turnLeft')" onmouseup="sendCommand('stop')" ontouchstart="sendCommand('turnLeft')" ontouchend="sendCommand('stop')">Drehe links</button>
    <button onmousedown="sendCommand('turnRight')" onmouseup="sendCommand('stop')" ontouchstart="sendCommand('turnRight')" ontouchend="sendCommand('stop')">Drehe rechts</button>
  </div>
  <div class="button-container">
    <button onmousedown="sendCommand('backward')" onmouseup="sendCommand('stop')" ontouchstart="sendCommand('backward')" ontouchend="sendCommand('stop')">Zurueck</button>
  </div>
  <div class="dropdown-container">
    <h3>Lichteffekt</h3>
    <select id="lightEffect" onchange="updateLightEffect(this.value)">
      <option value="normal">Normal</option>
      <option value="warning">Warnblinklicht</option>
      <option value="police">Polizei</option>
      <option value="off">Aus</option>
    </select>
  </div>
  <h3>Geschwindigkeit</h3>
  <input type="range" min="0" max="255" value="255" class="slider" id="speedSlider" oninput="updateSpeed(this.value)">
  <p>Aktuelle Geschwindigkeit: <span id="speedValue">255</span></p>
  <h3>LED-Helligkeit</h3>
  <input type="range" min="0" max="255" value="128" class="slider" id="brightnessSlider" oninput="updateBrightness(this.value)">
  <p>Aktuelle Helligkeit: <span id="brightnessValue">128</span></p>
  <div class="recorder-container">
    <h3>Streckenrecorder</h3>
    <button onclick="startRecording()">Aufnahme starten</button>
    <button onclick="stopRecording()">Aufnahme stoppen</button><br>
    <button onclick="playRecording()">Wiedergabe starten</button>
    <button onclick="pauseRecording()">Wiedergabe pausieren</button><br>
    <button onclick="deleteRecording()">Strecke loeschen</button>
    <button onclick="downloadLog()">Log herunterladen</button>
    <pre id="commandLog"></pre>
  </div>
  <p id="obstacleWarning" class="warning">Hindernis erkannt! Vorwärtsbewegung blockiert.</p>
  <p class="version">Softwareversion: %VERSION%</p>
  <script>
    let activeCommand = null; // Verfolgt den aktuellen Befehl, um Mehrfachanfragen zu verhindern
    function sendCommand(command) {
      if (activeCommand !== command) {
        activeCommand = command;
        fetch('/' + command)
          .then(response => {
            console.log('Befehl gesendet: ' + command, response.status);
            if (response.status === 403) {
              document.getElementById('obstacleWarning').style.display = 'block';
            } else {
              document.getElementById('obstacleWarning').style.display = 'none';
            }
            return response.text();
          })
          .then(text => console.log('Serverantwort: ' + text))
          .catch(error => console.error('Fehler bei Befehl ' + command + ':', error));
      }
    }
    function updateSpeed(value) {
      fetch('/speed?value=' + value).then(response => {
        document.getElementById('speedValue').innerText = value;
      });
    }
    function updateBrightness(value) {
      fetch('/brightness?value=' + value).then(response => {
        document.getElementById('brightnessValue').innerText = value;
      });
    }
    function updateLightEffect(effect) {
      fetch('/lighteffect?value=' + effect).then(response => {
        console.log('Lichteffekt gesetzt: ' + effect);
      });
    }
    function startRecording() {
      fetch('/record')
        .then(response => response.text())
        .then(data => console.log(data))
        .catch(error => console.error('Fehler:', error));
    }
    function stopRecording() {
      fetch('/stop_record')
        .then(response => response.text())
        .then(data => console.log(data))
        .catch(error => console.error('Fehler:', error));
    }
    function playRecording() {
      fetch('/play')
        .then(response => response.text())
        .then(data => console.log(data))
        .catch(error => console.error('Fehler:', error));
    }
    function pauseRecording() {
      fetch('/pause')
        .then(response => response.text())
        .then(data => console.log(data))
        .catch(error => console.error('Fehler:', error));
    }
    function deleteRecording() {
      fetch('/delete_log')
        .then(response => response.text())
        .then(data => {
          console.log(data);
          updateCommandLog(); // Log aktualisieren
        })
        .catch(error => console.error('Fehler:', error));
    }
    function downloadLog() {
      window.location.href = '/download';
    }
    function updateCommandLog() {
      fetch('/log')
        .then(response => response.text())
        .then(data => {
          document.getElementById('commandLog').innerText = data;
        });
      setTimeout(updateCommandLog, 5000); // Alle 5 Sekunden aktualisieren
    }
    updateCommandLog();
    // Tastatursteuerung
    document.addEventListener('keydown', (event) => {
      if (event.repeat) return; // Ignoriere wiederholte keydown-Events
      switch (event.key) {
        case 'ArrowUp':
          sendCommand('forward');
          break;
        case 'ArrowDown':
          sendCommand('backward');
          break;
        case 'ArrowLeft':
          sendCommand('turnLeft');
          break;
        case 'ArrowRight':
          sendCommand('turnRight');
          break;
      }
    });
    document.addEventListener('keyup', (event) => {
      if (['ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight'].includes(event.key)) {
        sendCommand('stop');
        activeCommand = null; // Setze den aktiven Befehl zurück
      }
    });
  </script>
</body>
</html>
)rawliteral";

void setup() {
  // Serielle Kommunikation für Debugging
  Serial.begin(115200);
  Serial.println("[SETUP] Initialisiere serielle Kommunikation mit 115200 Baud");

  // SPIFFS initialisieren
  if (!SPIFFS.begin(true)) {
    Serial.println("[SETUP] Fehler beim Mounten von SPIFFS");
    return;
  }
  Serial.println("[SETUP] SPIFFS gemountet");

  // Pins als Ausgang definieren
  Serial.println("[SETUP] Konfiguriere Pins für DRV8833-Module");
  pinMode(IN1_LEFT, OUTPUT);
  pinMode(IN2_LEFT, OUTPUT);
  pinMode(IN3_LEFT, OUTPUT);
  pinMode(IN4_LEFT, OUTPUT);
  
  pinMode(IN1_RIGHT, OUTPUT);
  pinMode(IN2_RIGHT, OUTPUT);
  pinMode(IN3_RIGHT, OUTPUT);
  pinMode(IN4_RIGHT, OUTPUT);

  // HC-SR04 Pins konfigurieren
  Serial.println("[SETUP] Konfiguriere Pins für HC-SR04");
  pinMode(TRIG_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);

  // Motoren initial stoppen
  Serial.println("[SETUP] Stoppe Motoren initial");
  stop();

  // LEDs initialisieren
  Serial.println("[SETUP] Initialisiere WS2812-LEDs");
  initLeds();
  setLedColors(); // LEDs standardmäßig ein

  // Mit WLAN verbinden
  Serial.print("[SETUP] Verbinde mit WLAN: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\n[SETUP] WLAN verbunden");
  Serial.print("[SETUP] IP-Adresse: ");
  Serial.println(WiFi.localIP());

  // mDNS starten
  Serial.println("[SETUP] Starte mDNS mit Hostname 'modellauto'");
  if (MDNS.begin("modellauto")) {
    Serial.println("[SETUP] mDNS gestartet: http://modellauto.local");
  } else {
    Serial.println("[SETUP] Fehler beim Starten von mDNS");
  }

  // ArduinoOTA initialisieren
  Serial.println("[SETUP] Initialisiere ArduinoOTA");
  ArduinoOTA.setHostname("modellauto");
  ArduinoOTA.setPassword("admin"); // Optional: Passwort für OTA-Updates

  ArduinoOTA.onStart([&]() {
    Serial.println("[OTA] Update gestartet");
    stop(); // Motoren während des Updates stoppen
  });
  ArduinoOTA.onEnd([&]() {
    Serial.println("[OTA] Update abgeschlossen");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("[OTA] Fortschritt: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("[OTA] Fehler[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Serial.println("Authentifizierung fehlgeschlagen");
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin fehlgeschlagen");
    else if (error == OTA_CONNECT_ERROR) Serial.println("Verbindung fehlgeschlagen");
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Empfang fehlgeschlagen");
    else if (error == OTA_END_ERROR) Serial.println("Ende fehlgeschlagen");
  });

  ArduinoOTA.begin();
  Serial.println("[SETUP] OTA bereit");

  // Webserver-Routen definieren
  Serial.println("[SETUP] Konfiguriere Webserver-Routen");
  server.on("/", HTTP_GET, [&]() {
    Serial.println("[WEBSERVER] Anfrage für Hauptseite (/): Sende HTML");
    String htmlWithVersion = String(html);
    htmlWithVersion.replace("%VERSION%", SOFTWARE_VERSION);
    server.send(200, "text/html", htmlWithVersion);
  });

  server.on("/forward", HTTP_GET, [&]() {
    if (obstacleDetected) {
      Serial.println("[WEBSERVER] Anfrage für /forward: Blockiert wegen Hindernis");
      server.send(403, "text/plain", "Vorwärts blockiert: Hindernis erkannt");
    } else {
      Serial.println("[WEBSERVER] Anfrage für /forward: Führe Vorwärtsbewegung aus");
      forward();
      logRecorderCommand("forward");
      server.send(200, "text/plain", "Vorwärts");
    }
  });

  server.on("/backward", HTTP_GET, [&]() {
    Serial.println("[WEBSERVER] Anfrage für /backward: Führe Rückwärtsbewegung aus");
    backward();
    logRecorderCommand("backward");
    server.send(200, "text/plain", "Rückwärts");
  });

  server.on("/turnRight", HTTP_GET, [&]() {
    Serial.println("[WEBSERVER] Anfrage für /turnRight: Führe Rechtsdrehung aus");
    turnRight();
    logRecorderCommand("turnRight");
    startBlinking("right");
    server.send(200, "text/plain", "Drehe rechts");
  });

  server.on("/turnLeft", HTTP_GET, [&]() {
    Serial.println("[WEBSERVER] Anfrage für /turnLeft: Führe Linksdrehung aus");
    turnLeft();
    logRecorderCommand("turnLeft");
    startBlinking("left");
    server.send(200, "text/plain", "Drehe links");
  });

  server.on("/stop", HTTP_GET, [&]() {
    Serial.println("[WEBSERVER] Anfrage für /stop: Stoppe Motoren");
    stop();
    logRecorderCommand("stop");
    stopBlinking();
    server.send(200, "text/plain", "Stop");
    Serial.println("[WEBSERVER] Stop-Befehl gesendet");
  });

  server.on("/speed", HTTP_GET, [&]() {
    if (server.hasArg("value")) {
      motorSpeed = server.arg("value").toInt();
      Serial.print("[WEBSERVER] Anfrage für /speed: Neue Geschwindigkeit = ");
      Serial.println(motorSpeed);
    } else {
      Serial.println("[WEBSERVER] Anfrage für /speed: Kein Wert angegeben");
    }
    server.send(200, "text/plain", "Geschwindigkeit: " + String(motorSpeed));
  });

  server.on("/brightness", HTTP_GET, [&]() {
    if (server.hasArg("value")) {
      int brightness = server.arg("value").toInt();
      Serial.print("[WEBSERVER] Anfrage für /brightness: Neue Helligkeit = ");
      Serial.println(brightness);
      setLedBrightness(brightness);
      server.send(200, "text/plain", "Helligkeit: " + String(brightness));
    } else {
      Serial.println("[WEBSERVER] Anfrage für /brightness: Kein Wert angegeben");
      server.send(400, "text/plain", "Kein Wert angegeben");
    }
  });

  server.on("/lighteffect", HTTP_GET, [&]() {
    if (server.hasArg("value")) {
      String effect = server.arg("value");
      if (effect == "normal" || effect == "warning" || effect == "police" || effect == "off") {
        Serial.print("[WEBSERVER] Anfrage für /lighteffect: Neuer Effekt = ");
        Serial.println(effect);
        setLightEffect(effect);
        server.send(200, "text/plain", "Lichteffekt: " + effect);
      } else {
        Serial.println("[WEBSERVER] Anfrage für /lighteffect: Ungültiger Effekt");
        server.send(400, "text/plain", "Ungültiger Effekt");
      }
    } else {
      Serial.println("[WEBSERVER] Anfrage für /lighteffect: Kein Wert angegeben");
      server.send(400, "text/plain", "Kein Wert angegeben");
    }
  });

  // Streckenrecorder-Routen
  handleRecorderRoutes();

  // Webserver starten
  Serial.println("[SETUP] Starte Webserver");
  server.begin();
  Serial.println("[SETUP] Webserver gestartet");
}

void loop() {
  server.handleClient();
  ArduinoOTA.handle();
  checkObstacle(); // Prüfe auf Hindernisse
  updateLeds(); // Aktualisiere Blinker
  updateRecorder(); // Aktualisiere Streckenrecorder
}

// Funktion zur Hinderniserkennung
void checkObstacle() {
  // Trigger-Puls senden
  digitalWrite(TRIG_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);

  // Echo-Zeit messen
  long duration = pulseIn(ECHO_PIN, HIGH, 30000); // Timeout nach 30ms
  float distance = duration * 0.034 / 2; // Entfernung in cm

  if (duration == 0 || distance > 400) {
    Serial.println("[SENSOR] Ungültige Messung oder kein Hindernis");
    distance = 400; // Max. Reichweite annehmen
  } else {
    Serial.print("[SENSOR] Entfernung: ");
    Serial.print(distance);
    Serial.println(" cm");
  }

  bool previousObstacleState = obstacleDetected;

  if (distance < OBSTACLE_DISTANCE && distance > 0) {
    obstacleCount++;
    Serial.print("[SENSOR] Hindernis-Messung #");
    Serial.println(obstacleCount);
    if (obstacleCount >= OBSTACLE_CONFIRM_COUNT) {
      obstacleDetected = true;
    }
  } else {
    obstacleCount = 0; // Zähler zurücksetzen
    obstacleDetected = false;
    Serial.println("[SENSOR] Kein Hindernis erkannt");
  }

  if (obstacleDetected && !previousObstacleState) {
    Serial.println("[SENSOR] Hindernis bestätigt! Notstopp ausgelöst");
    stop(); // Notstopp
    setEmergencyBlink(true); // Schnelles Blinken aktivieren
  } else if (!obstacleDetected && previousObstacleState) {
    Serial.println("[SENSOR] Hindernis entfernt");
    setEmergencyBlink(false); // Schnelles Blinken deaktivieren
  }
}

// Motorsteuerungsfunktionen
void forward() {
  Serial.print("[MOTOR] Führe Vorwärtsbewegung aus mit Geschwindigkeit: ");
  Serial.println(motorSpeed);
  // Linke Motoren vorwärts
  analogWrite(IN1_LEFT, motorSpeed);
  digitalWrite(IN2_LEFT, LOW);
  analogWrite(IN3_LEFT, motorSpeed);
  digitalWrite(IN4_LEFT, LOW);
  
  // Rechte Motoren vorwärts
  analogWrite(IN1_RIGHT, motorSpeed);
  digitalWrite(IN2_RIGHT, LOW);
  analogWrite(IN3_RIGHT, motorSpeed);
  digitalWrite(IN4_RIGHT, LOW);
}

void backward() {
  Serial.print("[MOTOR] Führe Rückwärtsbewegung aus mit Geschwindigkeit: ");
  Serial.println(motorSpeed);
  // Linke Motoren rückwärts
  digitalWrite(IN1_LEFT, LOW);
  analogWrite(IN2_LEFT, motorSpeed);
  digitalWrite(IN3_LEFT, LOW);
  analogWrite(IN4_LEFT, motorSpeed);
  
  // Rechte Motoren rückwärts
  digitalWrite(IN1_RIGHT, LOW);
  analogWrite(IN2_RIGHT, motorSpeed);
  digitalWrite(IN3_RIGHT, LOW);
  analogWrite(IN4_RIGHT, motorSpeed);
}

void turnLeft() {
  Serial.print("[MOTOR] Führe Rechtsdrehung aus mit Geschwindigkeit: ");
  Serial.println(motorSpeed);
  // Linke Motoren rückwärts
  digitalWrite(IN1_LEFT, LOW);
  analogWrite(IN2_LEFT, motorSpeed);
  digitalWrite(IN3_LEFT, LOW);
  analogWrite(IN4_LEFT, motorSpeed);
  
  // Rechte Motoren vorwärts
  analogWrite(IN1_RIGHT, motorSpeed);
  digitalWrite(IN2_RIGHT, LOW);
  analogWrite(IN3_RIGHT, motorSpeed);
  digitalWrite(IN4_RIGHT, LOW);
}

void turnRight() {
  Serial.print("[MOTOR] Führe Linksdrehung aus mit Geschwindigkeit: ");
  Serial.println(motorSpeed);
  // Linke Motoren vorwärts
  analogWrite(IN1_LEFT, motorSpeed);
  digitalWrite(IN2_LEFT, LOW);
  analogWrite(IN3_LEFT, motorSpeed);
  digitalWrite(IN4_LEFT, LOW);
  
  // Rechte Motoren rückwärts
  digitalWrite(IN1_RIGHT, LOW);
  analogWrite(IN2_RIGHT, motorSpeed);
  digitalWrite(IN3_RIGHT, LOW);
  analogWrite(IN4_RIGHT, motorSpeed);
}

void stop() {
  Serial.println("[MOTOR] Stoppe alle Motoren");
  // Alle Motoren stoppen (Freilauf)
  analogWrite(IN1_LEFT, 0);
  analogWrite(IN2_LEFT, 0);
  analogWrite(IN3_LEFT, 0);
  analogWrite(IN4_LEFT, 0);
  
  analogWrite(IN1_RIGHT, 0);
  analogWrite(IN2_RIGHT, 0);
  analogWrite(IN3_RIGHT, 0);
  analogWrite(IN4_RIGHT, 0);
  Serial.println("[MOTOR] Alle IN-Pins auf 0 gesetzt (PWM deaktiviert)");
}