#include <FastLED.h>

// LED-Konfiguration
#define NUM_LEDS 7 // Anzahl der LEDs pro Streifen
CRGB frontLeds[NUM_LEDS]; // Array für vordere LEDs
CRGB rearLeds[NUM_LEDS];  // Array für hintere LEDs
String lightEffect = "normal"; // Aktueller Lichteffekt ("normal", "warning", "police", "off")
String blinkerState = "none"; // Blinker-Status ("none", "right", "left")
unsigned long lastBlinkTime = 0; // Letzter Blink-Zeitpunkt
const unsigned long blinkInterval = 500; // Blink-Intervall für normale Blinker (ms)
bool blinkOn = false; // Blink-Status (an/aus)
int ledBrightness = 128; // Globale Variable für Helligkeit (0-255)
bool emergencyBlink = false; // Notstopp-Blinken aktiv
const unsigned long emergencyBlinkInterval = 200; // Schnelles Blink-Intervall (ms)
bool playbackBlink = false; // Wiedergabe-Blinken aktiv
const unsigned long playbackBlinkInterval = 300; // Blink-Intervall für Wiedergabe (ms)

void initLeds() {
  // Initialisiere FastLED für beide Streifen
  FastLED.addLeds<WS2812, LED_PIN_FRONT, GRB>(frontLeds, NUM_LEDS);
  FastLED.addLeds<WS2812, LED_PIN_REAR, GRB>(rearLeds, NUM_LEDS);
  
  // Setze Helligkeit
  FastLED.setBrightness(ledBrightness);
  
  Serial.println("[LED] WS2812-LEDs initialisiert");
}

void setLedColors() {
  // Standardmäßig alle LEDs ausschalten
  fill_solid(frontLeds, NUM_LEDS, CRGB::Black);
  fill_solid(rearLeds, NUM_LEDS, CRGB::Black);

  // Notstopp-Blinken hat höchste Priorität
  if (emergencyBlink) {
    if (blinkOn) {
      rearLeds[2] = CRGB::Red; // Mittlere hintere LED
      rearLeds[3] = CRGB::Red; // Mittlere hintere LED
      rearLeds[4] = CRGB::Red; // Mittlere hintere LED
      frontLeds[2] = CRGB::Red; // Links vorne
      frontLeds[3] = CRGB::Red; // Links vorne
      frontLeds[4] = CRGB::Red; // Links vorne
    }
    FastLED.show();
    Serial.println("[LED] Notstopp-Blinken aktiv");
    return;
  }

  // Wiedergabe-Blinken hat Priorität vor normalen Effekten
  if (playbackBlink) {
    if (blinkOn) {
      frontLeds[2] = CRGB::Yellow; // Mittlere vordere LEDs
      frontLeds[3] = CRGB::Yellow;
      frontLeds[4] = CRGB::Yellow;
      rearLeds[2] = CRGB::Yellow; // Mittlere hintere LEDs
      rearLeds[3] = CRGB::Yellow;
      rearLeds[4] = CRGB::Yellow;
    }
    FastLED.show();
    Serial.println("[LED] Wiedergabe-Blinken aktiv");
    return;
  }

  if (lightEffect == "off") {
    // Alle LEDs aus
    FastLED.show();
    Serial.println("[LED] Lichteffekt: Aus");
    return;
  }

  if (lightEffect == "normal") {
    // Normale Beleuchtung
    frontLeds[0] = CRGB::White;
    frontLeds[1] = CRGB::White;
    frontLeds[5] = CRGB::White;
    frontLeds[6] = CRGB::White;
    rearLeds[0] = CRGB::Red;
    rearLeds[1] = CRGB::Red;
    rearLeds[5] = CRGB::Red;
    rearLeds[6] = CRGB::Red;

    // Blinker-Logik nur im Normal-Modus
    if (blinkerState == "right" && blinkOn) {
      frontLeds[6] = CRGB::Yellow; // Blinker rechts vorne
      rearLeds[0] = CRGB::Yellow;  // Blinker rechts hinten
    } else if (blinkerState == "left" && blinkOn) {
      frontLeds[0] = CRGB::Yellow; // Blinker links vorne
      rearLeds[6] = CRGB::Yellow;  // Blinker links hinten
    }
  } else if (lightEffect == "warning") {
    // Warnblinklicht: Beide Blinker gleichzeitig
    if (blinkOn) {
      frontLeds[0] = CRGB::Yellow; // Blinker links vorne
      frontLeds[6] = CRGB::Yellow; // Blinker rechts vorne
      rearLeds[0] = CRGB::Yellow;  // Blinker rechts hinten
      rearLeds[6] = CRGB::Yellow;  // Blinker links hinten
    }
  } else if (lightEffect == "police") {
    // Polizei: Blaues Blinklicht, abwechselnd links/rechts
    if (blinkOn) {
      frontLeds[0] = CRGB::Blue; // Links vorne
      rearLeds[6] = CRGB::Blue;  // Links hinten
    } else {
      frontLeds[6] = CRGB::Blue; // Rechts vorne
      rearLeds[0] = CRGB::Blue;  // Rechts hinten
    }
  }
  
  // Aktualisiere LEDs
  FastLED.show();
  
  Serial.print("[LED] Farben gesetzt: ");
  if (lightEffect == "normal") {
    Serial.print("Normaler Modus, Blinker: ");
    Serial.println(blinkerState == "none" ? "Aus" : blinkerState);
  } else {
    Serial.print("Effekt: ");
    Serial.print(lightEffect);
    Serial.print(", Blink-Status: ");
    Serial.println(blinkOn ? "An" : "Aus");
  }
}

void setLedsState(bool state) {
  // Nicht mehr benötigt, da durch Lichteffekte ersetzt
}

void setLedBrightness(int brightness) {
  ledBrightness = constrain(brightness, 0, 255); // Begrenze auf 0-255
  FastLED.setBrightness(ledBrightness);
  setLedColors(); // Aktualisiere LEDs mit neuer Helligkeit
  Serial.print("[LED] Helligkeit gesetzt: ");
  Serial.println(ledBrightness);
}

void setLightEffect(String effect) {
  lightEffect = effect;
  blinkerState = "none"; // Blinker zurücksetzen bei Effektwechsel
  blinkOn = false;
  setLedColors();
  Serial.print("[LED] Lichteffekt gesetzt: ");
  Serial.println(effect);
}

void setEmergencyBlink(bool state) {
  emergencyBlink = state;
  blinkerState = "none"; // Blinker zurücksetzen
  blinkOn = false;
  lastBlinkTime = millis();
  setLedColors();
  Serial.print("[LED] Notstopp-Blinken: ");
  Serial.println(state ? "Aktiviert" : "Deaktiviert");
}

void setPlaybackBlink(bool state) {
  playbackBlink = state;
  blinkerState = "none"; // Blinker zurücksetzen
  blinkOn = false;
  lastBlinkTime = millis();
  setLedColors();
  Serial.print("[LED] Wiedergabe-Blinken: ");
  Serial.println(state ? "Aktiviert" : "Deaktiviert");
}

void startBlinking(String direction) {
  if (lightEffect == "normal" && !emergencyBlink && !playbackBlink && (direction == "right" || direction == "left")) {
    blinkerState = direction;
    lastBlinkTime = millis();
    blinkOn = true;
    setLedColors();
    Serial.print("[LED] Starte Blinker: ");
    Serial.println(direction);
  }
}

void stopBlinking() {
  if (lightEffect == "normal" && !emergencyBlink && !playbackBlink) {
    blinkerState = "none";
    blinkOn = false;
    setLedColors();
    Serial.println("[LED] Stoppe Blinker");
  }
}

void updateLeds() {
  unsigned long currentTime = millis();
  unsigned long interval = emergencyBlink ? emergencyBlinkInterval : (playbackBlink ? playbackBlinkInterval : blinkInterval);
  if (currentTime - lastBlinkTime >= interval) {
    blinkOn = !blinkOn;
    setLedColors();
    lastBlinkTime = currentTime;
  }
}