#include <FS.h>
#include <SPIFFS.h>
#include <WebServer.h>

// Externe Variablen aus main.ino
extern WebServer server;
extern bool obstacleDetected;
extern int motorSpeed;

// Funktionsprototypen aus main.ino
extern void forward();
extern void backward();
extern void turnLeft();
extern void turnRight();
extern void stop();

// Funktionsprototypen aus ledsteuerung.ino
extern void startBlinking(String direction);
extern void stopBlinking();
extern void setPlaybackBlink(bool state);

// Struktur für einen Befehl
struct DriveCommand {
  String command; // z. B. "forward", "backward", "turnLeft", "turnRight", "stop"
  unsigned long duration; // Dauer in Millisekunden
};

// Array für aufgezeichnete Befehle (max. 50)
#define MAX_COMMANDS 50
DriveCommand commandLog[MAX_COMMANDS];
int commandCount = 0;
bool isRecording = false;
unsigned long lastCommandTime = 0;
bool isPlaying = false;
int currentCommandIndex = 0;
unsigned long playStartTime = 0;
bool isPaused = false;
unsigned long playbackStartTime = 0; // Für Timeout

// Dateipfad für SPIFFS
const char* logFile = "/drive_log.txt";

void handleRecorderRoutes() {
  server.on("/record", HTTP_GET, [&]() {
    if (!isRecording && !isPlaying) {
      isRecording = true;
      commandCount = 0;
      lastCommandTime = millis();
      Serial.println("[RECORDER] Aufnahme gestartet");
      server.send(200, "text/plain", "Aufnahme gestartet");
    } else {
      server.send(400, "text/plain", "Aufnahme läuft oder Wiedergabe aktiv");
    }
  });

  server.on("/stop_record", HTTP_GET, [&]() {
    if (isRecording) {
      isRecording = false;
      if (commandCount > 0) {
        commandLog[commandCount-1].duration = millis() - lastCommandTime;
        saveToFile();
      }
      Serial.println("[RECORDER] Aufnahme gestoppt");
      server.send(200, "text/plain", "Aufnahme gestoppt");
    } else {
      server.send(400, "text/plain", "Keine Aufnahme aktiv");
    }
  });

  server.on("/play", HTTP_GET, [&]() {
    if (!isPlaying && !isRecording) {
      loadFromFile();
      if (commandCount == 0) {
        server.send(400, "text/plain", "Keine Befehle zum Abspielen");
        return;
      }
      isPlaying = true;
      isPaused = false;
      currentCommandIndex = 0;
      playStartTime = millis();
      playbackStartTime = millis(); // Timeout starten
      executeCommand(commandLog[0].command);
      setPlaybackBlink(true); // Aktiviere gelbes Blinken
      Serial.println("[RECORDER] Wiedergabe gestartet");
      server.send(200, "text/plain", "Wiedergabe gestartet");
    } else {
      server.send(400, "text/plain", "Wiedergabe läuft oder Aufnahme aktiv");
    }
  });

  server.on("/pause", HTTP_GET, [&]() {
    if (isPlaying) {
      isPaused = !isPaused;
      if (isPaused) {
        stop();
        setPlaybackBlink(false); // Deaktiviere gelbes Blinken
        Serial.println("[RECORDER] Wiedergabe pausiert");
        server.send(200, "text/plain", "Wiedergabe pausiert");
      } else {
        playStartTime = millis(); // Zeit zurücksetzen
        executeCommand(commandLog[currentCommandIndex].command);
        setPlaybackBlink(true); // Aktiviere gelbes Blinken
        Serial.println("[RECORDER] Wiedergabe fortgesetzt");
        server.send(200, "text/plain", "Wiedergabe fortgesetzt");
      }
    } else {
      server.send(400, "text/plain", "Keine Wiedergabe aktiv");
    }
  });

  server.on("/log", HTTP_GET, [&]() {
    String log = "";
    for (int i = 0; i < commandCount; i++) {
      log += commandLog[i].command + " (" + String(commandLog[i].duration) + "ms)\n";
    }
    server.send(200, "text/plain", log);
  });

  server.on("/download", HTTP_GET, [&]() {
    File file = SPIFFS.open(logFile, "r");
    if (!file) {
      server.send(404, "text/plain", "Datei nicht gefunden");
      return;
    }
    server.streamFile(file, "text/plain");
    file.close();
  });

  server.on("/delete_log", HTTP_GET, [&]() {
    // Stoppe laufende Prozesse
    if (isPlaying) {
      isPlaying = false;
      isPaused = false;
      stop();
      setPlaybackBlink(false); // Deaktiviere gelbes Blinken
      Serial.println("[RECORDER] Wiedergabe gestoppt für Löschung");
    }
    if (isRecording) {
      isRecording = false;
      Serial.println("[RECORDER] Aufnahme gestoppt für Löschung");
    }

    // Lösche Datei
    if (SPIFFS.exists(logFile)) {
      if (SPIFFS.remove(logFile)) {
        Serial.println("[RECORDER] Strecke gelöscht: " + String(logFile));
      } else {
        Serial.println("[RECORDER] Fehler beim Löschen von " + String(logFile));
        server.send(500, "text/plain", "Fehler beim Löschen der Strecke");
        return;
      }
    } else {
      Serial.println("[RECORDER] Keine Datei zum Löschen vorhanden");
    }

    // Setze alle Zustände zurück
    commandCount = 0;
    currentCommandIndex = 0;
    playStartTime = 0;
    lastCommandTime = 0;
    playbackStartTime = 0;
    Serial.println("[RECORDER] Aufzeichnungslogik vollständig zurückgesetzt");

    server.send(200, "text/plain", "Strecke gelöscht und Logik zurückgesetzt");
  });
}

void logRecorderCommand(String command) {
  if (isRecording) {
    if (commandCount >= MAX_COMMANDS) {
      Serial.println("[RECORDER] Max. Befehle erreicht");
      isRecording = false;
      if (commandCount > 0) {
        commandLog[commandCount-1].duration = millis() - lastCommandTime;
        saveToFile();
      }
      return;
    }

    unsigned long currentTime = millis();
    if (commandCount > 0) {
      commandLog[commandCount-1].duration = currentTime - lastCommandTime;
    }

    commandLog[commandCount].command = command;
    commandLog[commandCount].duration = 0; // Wird beim nächsten Befehl gesetzt
    commandCount++;
    lastCommandTime = currentTime;
    Serial.println("[RECORDER] Befehl aufgezeichnet: " + command);
  }
}

void executeCommand(String command) {
  if (command == "forward") {
    if (!obstacleDetected) {
      forward();
    }
  } else if (command == "backward") {
    backward();
  } else if (command == "turnLeft") {
    turnLeft();
    startBlinking("left");
  } else if (command == "turnRight") {
    turnRight();
    startBlinking("right");
  } else if (command == "stop") {
    stop();
    stopBlinking();
  }
  Serial.println("[RECORDER] Befehl ausgeführt: " + command);
}

void saveToFile() {
  File file = SPIFFS.open(logFile, "w");
  if (!file) {
    Serial.println("[SPIFFS] Fehler beim Öffnen der Datei zum Schreiben");
    return;
  }

  for (int i = 0; i < commandCount; i++) {
    file.println(commandLog[i].command + "," + String(commandLog[i].duration));
  }
  file.close();
  Serial.println("[SPIFFS] Befehle in " + String(logFile) + " gespeichert");
}

void loadFromFile() {
  File file = SPIFFS.open(logFile, "r");
  if (!file) {
    Serial.println("[SPIFFS] Fehler beim Öffnen der Datei zum Lesen");
    commandCount = 0;
    return;
  }

  commandCount = 0;
  while (file.available() && commandCount < MAX_COMMANDS) {
    String line = file.readStringUntil('\n');
    line.trim();
    int commaIndex = line.indexOf(',');
    if (commaIndex != -1) {
      String cmd = line.substring(0, commaIndex);
      if (cmd == "forward" || cmd == "backward" || cmd == "turnLeft" || 
          cmd == "turnRight" || cmd == "stop") {
        commandLog[commandCount].command = cmd;
        commandLog[commandCount].duration = line.substring(commaIndex + 1).toInt();
        commandCount++;
      }
    }
  }
  file.close();
  Serial.println("[SPIFFS] " + String(commandCount) + " Befehle aus " + String(logFile) + " geladen");
}

void updateRecorder() {
  if (!isPlaying) {
    return; // Frühzeitig abbrechen, wenn keine Wiedergabe aktiv
  }

  Serial.print("[RECORDER] Update: isPlaying=");
  Serial.print(isPlaying);
  Serial.print(", isPaused=");
  Serial.print(isPaused);
  Serial.print(", obstacleDetected=");
  Serial.print(obstacleDetected);
  Serial.print(", currentCommandIndex=");
  Serial.println(currentCommandIndex);

  // Berechne maximale Wiedergabedauer für Timeout
  unsigned long maxDuration = 0;
  for (int i = 0; i < commandCount; i++) {
    maxDuration += commandLog[i].duration;
  }
  if (millis() - playbackStartTime > maxDuration + 1000) {
    Serial.println("[RECORDER] Wiedergabe-Timeout erreicht, beende Wiedergabe");
    isPlaying = false;
    isPaused = false;
    stop();
    setPlaybackBlink(false);
    return;
  }

  if (isPlaying && !isPaused && !obstacleDetected && currentCommandIndex < commandCount) {
    unsigned long currentTime = millis();
    if (currentTime - playStartTime >= commandLog[currentCommandIndex].duration) {
      currentCommandIndex++;
      if (currentCommandIndex < commandCount) {
        executeCommand(commandLog[currentCommandIndex].command);
        playStartTime = currentTime;
      } else {
        stop();
        isPlaying = false;
        setPlaybackBlink(false); // Deaktiviere gelbes Blinken
        Serial.println("[RECORDER] Wiedergabe abgeschlossen");
      }
    }
  } else if (isPlaying && obstacleDetected && !isPaused) {
    stop();
    setPlaybackBlink(false); // Deaktiviere gelbes Blinken
    Serial.println("[RECORDER] Wiedergabe pausiert wegen Hindernis");
    isPaused = true;
  }
}